@@ -29,6 +29,15 @@ function octaviaclient_install {
     fi
 }
 
+function octavia_lib_install {
+    if use_library_from_git "octavia-lib"; then
+        git_clone_by_name "octavia-lib"
+        setup_dev_lib "octavia-lib"
+    else
+        pip_install_gr octavia-lib
+    fi
+}
+
 function install_diskimage_builder {
     if use_library_from_git "diskimage-builder"; then
         GITREPO["diskimage-builder"]=$DISKIMAGE_BUILDER_REPO_URL
@@ -49,6 +58,11 @@ function build_octavia_worker_image {
     # set up diskimage-builder if we need to
     install_diskimage_builder
 
+    # Pull in DIB local elements if they are defined in devstack
+    if [ -n "$DIB_LOCAL_ELEMENTS" ]; then
+        export DIB_LOCAL_ELEMENTS=$DIB_LOCAL_ELEMENTS
+    fi
+
     # pull the agent code from the current code zuul has a reference to
     if [ -n "$DIB_REPOLOCATION_pip_and_virtualenv" ]; then
         export DIB_REPOLOCATION_pip_and_virtualenv=$DIB_REPOLOCATION_pip_and_virtualenv
@@ -74,8 +88,15 @@ function build_octavia_worker_image {
     if [[ ${OCTAVIA_AMP_IMAGE_SIZE:+1} ]] ; then
     export PARAM_OCTAVIA_AMP_IMAGE_SIZE='-s '$OCTAVIA_AMP_IMAGE_SIZE
     fi
+
     if ! [ -f $OCTAVIA_AMP_IMAGE_FILE ]; then
-        $OCTAVIA_DIR/diskimage-create/diskimage-create.sh $octavia_dib_tracing_arg -o $OCTAVIA_AMP_IMAGE_FILE ${PARAM_OCTAVIA_AMP_BASE_OS:-} ${PARAM_OCTAVIA_AMP_DISTRIBUTION_RELEASE_ID:-} ${PARAM_OCTAVIA_AMP_IMAGE_SIZE:-}
+        local dib_logs=/var/log/dib-build
+        if [[ -e ${dib_logs} ]]; then
+            sudo rm -rf ${dib_logs}
+        fi
+        sudo mkdir -m755 ${dib_logs}
+        sudo chown $STACK_USER ${dib_logs}
+        $OCTAVIA_DIR/diskimage-create/diskimage-create.sh -l ${dib_logs}/$(basename $OCTAVIA_AMP_IMAGE_FILE).log $octavia_dib_tracing_arg -o $OCTAVIA_AMP_IMAGE_FILE ${PARAM_OCTAVIA_AMP_BASE_OS:-} ${PARAM_OCTAVIA_AMP_DISTRIBUTION_RELEASE_ID:-} ${PARAM_OCTAVIA_AMP_IMAGE_SIZE:-}
     fi
     upload_image file://${OCTAVIA_AMP_IMAGE_FILE} $TOKEN
 
@@ -147,8 +168,8 @@ function _start_octavia_apache_wsgi {
         enable_apache_site octavia-wsgi
     else
         enable_apache_site octavia
-        restart_apache_server
     fi
+    restart_apache_server
 }
 
 function _stop_octavia_apache_wsgi {
@@ -157,8 +178,8 @@ function _stop_octavia_apache_wsgi {
         stop_process o-api
     else
         disable_apache_site octavia
-        restart_apache_server
     fi
+    restart_apache_server
 }
 
 function create_octavia_accounts {
@@ -171,7 +192,13 @@ function create_octavia_accounts {
     local octavia_service=$(get_or_create_service "octavia" \
         $OCTAVIA_SERVICE_TYPE "Octavia Load Balancing Service")
 
-    if [[ "$WSGI_MODE" == "uwsgi" ]]; then
+    if [[ "$WSGI_MODE" == "uwsgi" ]] && [[ "$OCTAVIA_NODE" == "main" ]] ; then
+        get_or_create_endpoint $octavia_service \
+            "$REGION_NAME" \
+            "$OCTAVIA_PROTOCOL://$SERVICE_HOST:$OCTAVIA_PORT/$OCTAVIA_SERVICE_TYPE" \
+            "$OCTAVIA_PROTOCOL://$SERVICE_HOST:$OCTAVIA_PORT/$OCTAVIA_SERVICE_TYPE" \
+            "$OCTAVIA_PROTOCOL://$SERVICE_HOST:$OCTAVIA_PORT/$OCTAVIA_SERVICE_TYPE"
+    elif [[ "$WSGI_MODE" == "uwsgi" ]]; then
         get_or_create_endpoint $octavia_service \
             "$REGION_NAME" \
             "$OCTAVIA_PROTOCOL://$SERVICE_HOST/$OCTAVIA_SERVICE_TYPE" \
@@ -191,15 +218,24 @@ function octavia_configure {
     sudo mkdir -m 755 -p $OCTAVIA_CONF_DIR
     safe_chown $STACK_USER $OCTAVIA_CONF_DIR
 
+    sudo mkdir -m 700 -p $OCTAVIA_RUN_DIR
+    safe_chown $STACK_USER $OCTAVIA_RUN_DIR
+
     if ! [ -e $OCTAVIA_CONF ] ; then
         cp $OCTAVIA_DIR/etc/octavia.conf $OCTAVIA_CONF
     fi
 
+    if ! [ -e $OCTAVIA_AUDIT_MAP ] ; then
+        cp $OCTAVIA_DIR/etc/audit/octavia_api_audit_map.conf.sample $OCTAVIA_AUDIT_MAP
+    fi
+
     # Use devstack logging configuration
     setup_logging $OCTAVIA_CONF
+    iniset $OCTAVIA_CONF DEFAULT debug $ENABLE_DEBUG_LOG_LEVEL
 
     # Change bind host
-    iniset $OCTAVIA_CONF DEFAULT bind_host $SERVICE_HOST
+    iniset $OCTAVIA_CONF api_settings bind_host $SERVICE_HOST
+    iniset $OCTAVIA_CONF api_settings api_handler queue_producer
 
     iniset $OCTAVIA_CONF database connection "mysql+pymysql://${DATABASE_USER}:${DATABASE_PASSWORD}@${DATABASE_HOST}:3306/octavia"
 
@@ -229,8 +265,6 @@ function octavia_configure {
     iniset $OCTAVIA_CONF house_keeping amphora_expiry_age ${OCTAVIA_AMP_EXPIRY_AGE}
     iniset $OCTAVIA_CONF house_keeping load_balancer_expiry_age ${OCTAVIA_LB_EXPIRY_AGE}
 
-    iniset $OCTAVIA_CONF DEFAULT api_handler queue_producer
-
     iniset $OCTAVIA_CONF DEFAULT transport_url $(get_transport_url)
 
     iniset $OCTAVIA_CONF oslo_messaging rpc_thread_pool_size 2
@@ -324,19 +358,14 @@ function octavia_configure {
     if [ $OCTAVIA_NODE == 'main' ]; then
         configure_octavia_api_haproxy
         # make sure octavia is reachable from haproxy
-        iniset $OCTAVIA_CONF DEFAULT bind_port ${OCTAVIA_HA_PORT}
-        iniset $OCTAVIA_CONF DEFAULT bind_host 0.0.0.0
+        iniset $OCTAVIA_CONF api_settings bind_port ${OCTAVIA_HA_PORT}
+        iniset $OCTAVIA_CONF api_settings bind_host 0.0.0.0
     fi
     if [ $OCTAVIA_NODE != 'main' ] && [ $OCTAVIA_NODE != 'standalone' ] ; then
         # make sure octavia is reachable from haproxy from main node
-        iniset $OCTAVIA_CONF DEFAULT bind_port ${OCTAVIA_HA_PORT}
-        iniset $OCTAVIA_CONF DEFAULT bind_host 0.0.0.0
+        iniset $OCTAVIA_CONF api_settings bind_port ${OCTAVIA_HA_PORT}
+        iniset $OCTAVIA_CONF api_settings bind_host 0.0.0.0
     fi
-
-    # set default graceful_shutdown_timeout to 300 sec (5 minutes)
-    # TODO(gthiemonge) update this value after persistant taskflow commits are
-    # merged
-    iniset $OCTAVIA_CONF DEFAULT graceful_shutdown_timeout 300
 }
 
 function create_mgmt_network_interface {
@@ -381,8 +410,8 @@ function create_mgmt_network_interface {
 
 function build_mgmt_network {
     # Create network and attach a subnet
-    OCTAVIA_AMP_NETWORK_ID=$(openstack network create lb-mgmt-net -f value -c id)
-    OCTAVIA_AMP_SUBNET_ID=$(openstack subnet create --subnet-range $OCTAVIA_MGMT_SUBNET --allocation-pool start=$OCTAVIA_MGMT_SUBNET_START,end=$OCTAVIA_MGMT_SUBNET_END --network lb-mgmt-net lb-mgmt-subnet -f value -c id)
+    openstack network create lb-mgmt-net
+    openstack subnet create --subnet-range $OCTAVIA_MGMT_SUBNET --allocation-pool start=$OCTAVIA_MGMT_SUBNET_START,end=$OCTAVIA_MGMT_SUBNET_END --network lb-mgmt-net lb-mgmt-subnet
 
     # Create security group and rules
     openstack security group create lb-mgmt-sec-grp
@@ -439,7 +468,7 @@ function configure_octavia_api_haproxy {
        DATA=(${NODE//:/ })
        NAME=$(echo -e "${DATA[0]}" | tr -d '[[:space:]]')
        IP=$(echo -e "${DATA[1]}" | tr -d '[[:space:]]')
-       echo "   server octavia-${NAME} ${IP}:${OCTAVIA_HA_PORT} weight 1" >> ${OCTAVIA_CONF_DIR}/haproxy.cfg
+       echo "   server octavia-${NAME} ${IP}:80 weight 1" >> ${OCTAVIA_CONF_DIR}/haproxy.cfg
     done
 
 }
@@ -460,6 +489,7 @@ function octavia_start {
         run_process $OCTAVIA_API  "$OCTAVIA_API_BINARY $OCTAVIA_API_ARGS"
     fi
 
+    run_process $OCTAVIA_DRIVER_AGENT "$OCTAVIA_DRIVER_AGENT_BINARY $OCTAVIA_DRIVER_AGENT_ARGS"
     run_process $OCTAVIA_CONSUMER  "$OCTAVIA_CONSUMER_BINARY $OCTAVIA_CONSUMER_ARGS"
     run_process $OCTAVIA_HOUSEKEEPER  "$OCTAVIA_HOUSEKEEPER_BINARY $OCTAVIA_HOUSEKEEPER_ARGS"
     run_process $OCTAVIA_HEALTHMANAGER  "$OCTAVIA_HEALTHMANAGER_BINARY $OCTAVIA_HEALTHMANAGER_ARGS"
@@ -472,6 +502,7 @@ function octavia_stop {
     else
         stop_process $OCTAVIA_API
     fi
+    stop_process $OCTAVIA_DRIVER_AGENT
     stop_process $OCTAVIA_CONSUMER
     stop_process $OCTAVIA_HOUSEKEEPER
     stop_process $OCTAVIA_HEALTHMANAGER
@@ -506,6 +537,9 @@ function octavia_cleanup {
     if [ ${OCTAVIA_CONF_DIR}x != x ] ; then
          sudo rm -rf ${OCTAVIA_CONF_DIR}
     fi
+    if [ ${OCTAVIA_RUN_DIR}x != x ] ; then
+         sudo rm -rf ${OCTAVIA_RUN_DIR}
+    fi
     if [ ${OCTAVIA_AMP_SSH_KEY_PATH}x != x ] ; then
         rm -f ${OCTAVIA_AMP_SSH_KEY_PATH} ${OCTAVIA_AMP_SSH_KEY_PATH}.pub
     fi
@@ -534,8 +568,10 @@ function octavia_init {
    if [ $OCTAVIA_NODE != 'main' ] && [ $OCTAVIA_NODE != 'standalone' ]  && [ $OCTAVIA_NODE != 'api' ]; then
        # without the other services enabled apparently we don't have
        # credentials at this point
-       TOP_DIR=$(cd $(dirname "$0") && pwd)
+#       TOP_DIR=$(cd $(dirname "$0") && pwd)
        source ${TOP_DIR}/openrc admin admin
+       OCTAVIA_AMP_NETWORK_ID=$(openstack network show lb-mgmt-net -f value -c id)
+       iniset $OCTAVIA_CONF controller_worker amp_boot_network_list ${OCTAVIA_AMP_NETWORK_ID}
    fi
 
    if [ $OCTAVIA_NODE == 'main' ] || [ $OCTAVIA_NODE == 'standalone' ] ; then
@@ -599,6 +635,10 @@ function octavia_init {
    fi
 }
 
+function _configure_tempest {
+    iniset $TEMPEST_CONFIG service_available octavia "True"
+}
+
 # check for service enabled
 if is_service_enabled $OCTAVIA; then
     if [ $OCTAVIA_NODE == 'main' ] || [ $OCTAVIA_NODE == 'standalone' ] ; then # main-ha node stuff only
@@ -616,6 +656,7 @@ if is_service_enabled $OCTAVIA; then
     if [[ "$1" == "stack" && "$2" == "install" ]]; then
         # Perform installation of service source
         echo_summary "Installing octavia"
+        octavia_lib_install
         octavia_install
         octaviaclient_install
 
@@ -632,6 +673,11 @@ if is_service_enabled $OCTAVIA; then
 
         echo_summary "Starting Octavia"
         octavia_start
+    elif [[ "$1" == "stack" && "$2" == "test-config" ]]; then
+        if is_service_enabled tempest; then
+            # Configure Tempest for Congress
+            _configure_tempest
+        fi
     fi
 fi
 
