@@ -17,12 +17,15 @@ import copy
 from oslo_log import log as logging
 from pecan import request
 import sqlalchemy
+from sqlalchemy.orm import aliased
 import sqlalchemy.sql as sa_sql
 
 from octavia.api.common import types
 from octavia.common.config import cfg
 from octavia.common import constants
 from octavia.common import exceptions
+from octavia.db import base_models
+from octavia.db import models
 
 CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
@@ -33,6 +36,10 @@ class PaginationHelper(object):
 
     Pass this class to `db.repositories` to apply it on query
     """
+    _auxiliary_arguments = ('limit', 'marker',
+                            'sort', 'sort_key', 'sort_dir',
+                            'fields', 'page_reverse',
+                            )
 
     def __init__(self, params, sort_dir=constants.DEFAULT_SORT_DIR):
         """Pagination Helper takes params and a default sort direction
@@ -50,6 +57,7 @@ class PaginationHelper(object):
         self.sort_keys = self._parse_sort_keys(params)
         self.params = params
         self.filters = None
+        self.page_reverse = params.get('page_reverse', 'False')
 
     @staticmethod
     def _parse_limit(params):
@@ -149,6 +157,7 @@ class PaginationHelper(object):
             next_attr = copy.copy(prev_attr)
             if self.marker:
                 prev_attr.append("marker={}".format(model_list[0].get('id')))
+                prev_attr.append("page_reverse=True")
                 prev_link = {
                     "rel": "previous",
                     "href": "{url}?{params}".format(
@@ -171,7 +180,55 @@ class PaginationHelper(object):
         links = [types.PageType(**link) for link in links]
         return links
 
-    def apply(self, query, model):
+    def _apply_tags_filtering(self, params, model, query):
+        if not getattr(model, "_tags", None):
+            return query
+
+        tag_alias = aliased(base_models.Tags)
+
+        if 'tags' in params:
+            tags = params.pop('tags')
+            if not isinstance(tags, list):
+                tags = [tags]
+            first_tag = tags.pop(0)
+            query = query.join(model._tags)
+            query = query.filter(base_models.Tags.tag == first_tag)
+
+            for tag in tags:
+                query = query.join(tag_alias, model._tags)
+                query = query.filter(tag_alias.tag == tag)
+
+        if 'tags-any' in params:
+            tags = params.pop('tags-any')
+            if not isinstance(tags, list):
+                tags = [tags]
+            query = query.join(tag_alias, model._tags)
+            query = query.filter(tag_alias.tag.in_(tags))
+
+        if 'not-tags' in params:
+            tags = params.pop('not-tags')
+            if not isinstance(tags, list):
+                tags = [tags]
+            first_tag = tags.pop(0)
+            subq = query.session.query(base_models.Tags.resource_id)
+            subq = subq.join(model._tags)
+            subq = subq.filter(base_models.Tags.tag == first_tag)
+            for tag in tags:
+                subq = subq.join(tag_alias, model._tags)
+                subq = subq.filter(tag_alias.tag == tag)
+
+            query = query.filter(~model.id.in_(subq))
+
+        if 'not-tags-any' in params:
+            tags = params.pop('not-tags-any')
+            if not isinstance(tags, list):
+                tags = [tags]
+            query = query.filter(
+                ~model._tags.any(base_models.Tags.tag.in_(tags)))
+
+        return query
+
+    def apply(self, query, model, enforce_valid_params=True):
         """Returns a query with sorting / pagination criteria added.
 
         Pagination works by requiring a unique sort_key specified by sort_keys.
@@ -189,6 +246,7 @@ class PaginationHelper(object):
         :param query: the query object to which we should add
         paging/sorting/filtering
         :param model: the ORM model class
+        :param enforce_valid_params: check for invalid enteries in self.params
 
         :rtype: sqlalchemy.orm.query.Query
         :returns: The query with sorting/pagination/filtering added.
@@ -196,15 +254,38 @@ class PaginationHelper(object):
 
         # Add filtering
         if CONF.api_settings.allow_filtering:
-            filter_attrs = [attr for attr in dir(
-                model.__v2_wsme__
-            ) if not callable(
-                getattr(model.__v2_wsme__, attr)
-            ) and not attr.startswith("_")]
-            self.filters = {k: v for (k, v) in self.params.items()
-                            if k in filter_attrs}
+            # Exclude (valid) arguments that are not used for data filtering
+            filter_params = {k: v for k, v in self.params.items()
+                             if k not in self._auxiliary_arguments}
+
+            secondary_query_filter = filter_params.pop(
+                "project_id", None) if (model == models.Amphora) else None
+
+            # Tranlate arguments from API standard to data model's field name
+            filter_params = (
+                model.__v2_wsme__.translate_dict_keys_to_data_model(
+                    filter_params)
+            )
+            if 'loadbalancer_id' in filter_params:
+                filter_params['load_balancer_id'] = filter_params.pop(
+                    'loadbalancer_id')
+
+            # Apply tags filtering for the models which support tags.
+            query = self._apply_tags_filtering(filter_params, model, query)
+
+            # Drop invalid arguments
+            self.filters = {k: v for (k, v) in filter_params.items()
+                            if k in vars(model.__data_model__())}
+
+            if enforce_valid_params and (
+                len(self.filters) < len(filter_params)
+            ):
+                raise exceptions.InvalidFilterArgument()
 
             query = model.apply_filter(query, model, self.filters)
+            if secondary_query_filter is not None:
+                query = query.filter(model.load_balancer.has(
+                    project_id=secondary_query_filter))
 
         # Add sorting
         if CONF.api_settings.allow_sorting:
@@ -261,9 +342,15 @@ class PaginationHelper(object):
                           model_attr), ], else_=default)
                     this_sort_dir = self.sort_keys[i][1]
                     if this_sort_dir == constants.DESC:
-                        crit_attrs.append((attr < marker_values[i]))
+                        if self.page_reverse == "True":
+                            crit_attrs.append((attr > marker_values[i]))
+                        else:
+                            crit_attrs.append((attr < marker_values[i]))
                     elif this_sort_dir == constants.ASC:
-                        crit_attrs.append((attr > marker_values[i]))
+                        if self.page_reverse == "True":
+                            crit_attrs.append((attr < marker_values[i]))
+                        else:
+                            crit_attrs.append((attr > marker_values[i]))
                     else:
                         raise exceptions.InvalidSortDirection(
                             key=this_sort_dir)
