@@ -14,6 +14,8 @@
 #
 
 import concurrent.futures
+import datetime
+
 from oslo_config import cfg
 from taskflow import engines as tf_engines
 
@@ -29,13 +31,15 @@ class BaseTaskFlowEngine(object):
     """
 
     def __init__(self):
+        # work around for https://bugs.python.org/issue7980
+        datetime.datetime.strptime('2014-06-19 22:47:16', '%Y-%m-%d %H:%M:%S')
         self.executor = concurrent.futures.ThreadPoolExecutor(
             max_workers=CONF.task_flow.max_workers)
 
     def _taskflow_load(self, flow, **kwargs):
         eng = tf_engines.load(
             flow,
-            engine_conf=CONF.task_flow.engine,
+            engine=CONF.task_flow.engine,
             executor=self.executor,
             never_resolve=CONF.task_flow.disable_revert,
             **kwargs)
