@@ -17,6 +17,7 @@
 Routines for configuring Octavia
 """
 
+import os
 import sys
 
 from keystoneauth1 import loading as ks_loading
@@ -33,37 +34,12 @@ from octavia import version
 
 LOG = logging.getLogger(__name__)
 
-# TODO(rm_work) Remove in or after "R" release
-API_SETTINGS_DEPRECATION_MESSAGE = _(
-    'This setting has moved to the [api_settings] section.')
 
 core_opts = [
     cfg.HostnameOpt('host', default=utils.get_hostname(),
                     help=_("The hostname Octavia is running on")),
     cfg.StrOpt('octavia_plugins', default='hot_plug_plugin',
                help=_("Name of the controller plugin to use")),
-
-    # TODO(johnsom) Remove in or after "R" release
-    cfg.IPOpt('bind_host', help=_("The host IP to bind to"),
-              deprecated_for_removal=True,
-              deprecated_reason=API_SETTINGS_DEPRECATION_MESSAGE),
-    # TODO(johnsom) Remove in or after "R" release
-    cfg.PortOpt('bind_port', help=_("The port to bind to"),
-                deprecated_for_removal=True,
-                deprecated_reason=API_SETTINGS_DEPRECATION_MESSAGE),
-    # TODO(johnsom) Remove in or after "R" release
-    cfg.StrOpt('auth_strategy',
-               choices=[constants.NOAUTH,
-                        constants.KEYSTONE,
-                        constants.TESTING],
-               help=_("The auth strategy for API requests."),
-               deprecated_for_removal=True,
-               deprecated_reason=API_SETTINGS_DEPRECATION_MESSAGE),
-    # TODO(johnsom) Remove in or after "R" release
-    cfg.StrOpt('api_handler',
-               help=_("The handler that the API communicates with"),
-               deprecated_for_removal=True,
-               deprecated_reason=API_SETTINGS_DEPRECATION_MESSAGE),
 ]
 
 api_opts = [
@@ -179,14 +155,19 @@ healthmanager_opts = [
               help=_('IP address the controller will listen on for '
                      'heart beats')),
     cfg.PortOpt('bind_port', default=5555,
-                help=_('Port number the controller will listen on'
+                help=_('Port number the controller will listen on '
                        'for heart beats')),
     cfg.IntOpt('failover_threads',
                default=10,
                help=_('Number of threads performing amphora failovers.')),
+    # TODO(tatsuma) Remove in or after "T" release
     cfg.IntOpt('status_update_threads',
                default=None,
-               help=_('Number of processes for amphora status update.')),
+               help=_('Number of processes for amphora status update.'),
+               deprecated_for_removal=True,
+               deprecated_reason=_('This option is replaced as '
+                                   'health_update_threads and '
+                                   'stats_update_threads')),
     cfg.IntOpt('health_update_threads',
                default=None,
                help=_('Number of processes for amphora health update.')),
@@ -194,7 +175,8 @@ healthmanager_opts = [
                default=None,
                help=_('Number of processes for amphora stats update.')),
     cfg.StrOpt('heartbeat_key',
-               help=_('key used to validate amphora sending'
+               mutable=True,
+               help=_('key used to validate amphora sending '
                       'the message'), secret=True),
     cfg.IntOpt('heartbeat_timeout',
                default=60,
@@ -211,9 +193,11 @@ healthmanager_opts = [
                 help=_('List of controller ip and port pairs for the '
                        'heartbeat receivers. Example 127.0.0.1:5555, '
                        '192.168.0.1:5555'),
+                mutable=True,
                 default=[]),
     cfg.IntOpt('heartbeat_interval',
                default=10,
+               mutable=True,
                help=_('Sleep time between sending heartbeats.')),
 
     # Used for updating health and stats
@@ -239,8 +223,8 @@ oslo_messaging_opts = [
                default='neutron_lbaas_event',
                help=_('topic name for communicating events through a queue')),
     cfg.StrOpt('event_stream_transport_url', default=None,
-               help=_('Transport URL to use for the neutron-lbaas'
-                      'synchronization event stream when neutron and octavia'
+               help=_('Transport URL to use for the neutron-lbaas '
+                      'synchronization event stream when neutron and octavia '
                       'have separate queues.')),
 ]
 
@@ -255,7 +239,7 @@ haproxy_amphora_opts = [
     cfg.BoolOpt('connection_logging', default=True,
                 help=_('Set this to False to disable connection logging.')),
     cfg.IntOpt('connection_max_retries',
-               default=300,
+               default=120,
                help=_('Retry threshold for connecting to amphorae.')),
     cfg.IntOpt('connection_retry_interval',
                default=5,
@@ -270,10 +254,10 @@ haproxy_amphora_opts = [
                       'seconds for active amphora.')),
     cfg.IntOpt('build_rate_limit',
                default=-1,
-               help=_('Number of amphorae that could be built per controller'
+               help=_('Number of amphorae that could be built per controller '
                       'worker, simultaneously.')),
     cfg.IntOpt('build_active_retries',
-               default=300,
+               default=120,
                help=_('Retry threshold for waiting for a build slot for '
                       'an amphorae.')),
     cfg.IntOpt('build_retry_interval',
@@ -305,6 +289,18 @@ haproxy_amphora_opts = [
     cfg.FloatOpt('rest_request_read_timeout', default=60,
                  help=_("The time in seconds to wait for a REST API "
                         "response.")),
+    cfg.IntOpt('timeout_client_data',
+               default=constants.DEFAULT_TIMEOUT_CLIENT_DATA,
+               help=_('Frontend client inactivity timeout.')),
+    cfg.IntOpt('timeout_member_connect',
+               default=constants.DEFAULT_TIMEOUT_MEMBER_CONNECT,
+               help=_('Backend member connection timeout.')),
+    cfg.IntOpt('timeout_member_data',
+               default=constants.DEFAULT_TIMEOUT_MEMBER_DATA,
+               help=_('Backend member inactivity timeout.')),
+    cfg.IntOpt('timeout_tcp_inspect',
+               default=constants.DEFAULT_TIMEOUT_TCP_INSPECT,
+               help=_('Time to wait for TCP packets for content inspection.')),
     # REST client
     cfg.StrOpt('client_cert', default='/etc/octavia/certs/client.pem',
                help=_("The client certificate to talk to the agent")),
@@ -322,7 +318,7 @@ controller_worker_opts = [
                default=1, min=1,
                help='Number of workers for the controller-worker service.'),
     cfg.IntOpt('amp_active_retries',
-               default=10,
+               default=30,
                help=_('Retry attempts to wait for Amphora to become active')),
     cfg.IntOpt('amp_active_wait_sec',
                default=10,
@@ -348,7 +344,8 @@ controller_worker_opts = [
                       'owner ID.  This is a recommended security setting.')),
     cfg.StrOpt('amp_ssh_key_name',
                default='',
-               help=_('SSH key name used to boot the Amphora')),
+               help=_('Optional SSH keypair name, in nova, that will be used '
+                      'for the authorized_keys inside the amphora.')),
     cfg.BoolOpt('amp_ssh_access_allowed',
                 default=True,
                 deprecated_for_removal=True,
@@ -384,6 +381,7 @@ controller_worker_opts = [
     cfg.StrOpt('loadbalancer_topology',
                default=constants.TOPOLOGY_SINGLE,
                choices=constants.SUPPORTED_LB_TOPOLOGIES,
+               mutable=True,
                help=_('Load balancer topology configuration. '
                       'SINGLE - One amphora per load balancer. '
                       'ACTIVE_STANDBY - Two amphora per load balancer.')),
@@ -421,7 +419,7 @@ certificate_opts = [
                default='barbican_acl_auth',
                help='Name of the Barbican authentication method to use'),
     cfg.StrOpt('service_name',
-               help=_('The name of the certificate service in the keystone'
+               help=_('The name of the certificate service in the keystone '
                       'catalog')),
     cfg.StrOpt('endpoint', help=_('A new endpoint to override the endpoint '
                                   'in the keystone catalog.')),
@@ -588,6 +586,51 @@ quota_opts = [
                help=_('Default per project health monitor quota.')),
 ]
 
+audit_opts = [
+    cfg.BoolOpt('enabled', default=False,
+                help=_('Enable auditing of API requests')),
+    cfg.StrOpt('audit_map_file',
+               default='/etc/octavia/octavia_api_audit_map.conf',
+               help=_('Path to audit map file for octavia-api service. '
+                      'Used only when API audit is enabled.')),
+    cfg.StrOpt('ignore_req_list', default='',
+               help=_('Comma separated list of REST API HTTP methods to be '
+                      'ignored during audit. For example: auditing will not '
+                      'be done on any GET or POST requests if this is set to '
+                      '"GET,POST". It is used only when API audit is '
+                      'enabled.')),
+]
+
+driver_agent_opts = [
+    cfg.StrOpt('status_socket_path',
+               default='/var/run/octavia/status.sock',
+               help=_('Path to the driver status unix domain socket file.')),
+    cfg.StrOpt('stats_socket_path',
+               default='/var/run/octavia/stats.sock',
+               help=_('Path to the driver statistics unix domain socket '
+                      'file.')),
+    cfg.IntOpt('status_request_timeout',
+               default=5,
+               help=_('Time, in seconds, to wait for a status update '
+                      'request.')),
+    cfg.IntOpt('status_max_processes',
+               default=50,
+               help=_('Maximum number of concurrent processes to use '
+                      'servicing status updates.')),
+    cfg.IntOpt('stats_request_timeout',
+               default=5,
+               help=_('Time, in seconds, to wait for a statistics update '
+                      'request.')),
+    cfg.IntOpt('stats_max_processes',
+               default=50,
+               help=_('Maximum number of concurrent processes to use '
+                      'servicing statistics updates.')),
+    cfg.FloatOpt('max_process_warning_percent',
+                 default=0.75, min=0.01, max=0.99,
+                 help=_('Percentage of max_processes (both status and stats) '
+                        'in use to start logging warning messages about an '
+                        'overloaded driver-agent.')),
+]
 
 # Register the configuration options
 cfg.CONF.register_opts(core_opts)
@@ -608,7 +651,8 @@ cfg.CONF.register_opts(nova_opts, group='nova')
 cfg.CONF.register_opts(glance_opts, group='glance')
 cfg.CONF.register_opts(neutron_opts, group='neutron')
 cfg.CONF.register_opts(quota_opts, group='quotas')
-
+cfg.CONF.register_opts(audit_opts, group='audit')
+cfg.CONF.register_opts(driver_agent_opts, group='driver_agent')
 
 cfg.CONF.register_opts(local.certgen_opts, group='certificates')
 cfg.CONF.register_opts(local.certmgr_opts, group='certificates')
@@ -632,6 +676,7 @@ def init(args, **kwargs):
              version='%%prog %s' % version.version_info.release_string(),
              **kwargs)
     handle_deprecation_compatibility()
+    setup_remote_pydev_debug()
 
 
 def setup_logging(conf):
@@ -653,22 +698,7 @@ def setup_logging(conf):
 # a value set in the previous location while allowing settings that have
 # not yet been moved to be utilized.
 def handle_deprecation_compatibility():
-    # TODO(johnsom) Remove in or after "R" release
-    if cfg.CONF.bind_host is not None:
-        cfg.CONF.set_default('bind_host', cfg.CONF.bind_host,
-                             group='api_settings')
-    # TODO(johnsom) Remove in or after "R" release
-    if cfg.CONF.bind_port is not None:
-        cfg.CONF.set_default('bind_port', cfg.CONF.bind_port,
-                             group='api_settings')
-    # TODO(johnsom) Remove in or after "R" release
-    if cfg.CONF.auth_strategy is not None:
-        cfg.CONF.set_default('auth_strategy', cfg.CONF.auth_strategy,
-                             group='api_settings')
-    # TODO(johnsom) Remove in or after "R" release
-    if cfg.CONF.api_handler is not None:
-        cfg.CONF.set_default('api_handler', cfg.CONF.api_handler,
-                             group='api_settings')
+    # TODO(tatsuma) Remove in or after "T" release
     if cfg.CONF.health_manager.status_update_threads is not None:
         cfg.CONF.set_default('health_update_threads',
                              cfg.CONF.health_manager.status_update_threads,
@@ -676,3 +706,34 @@ def handle_deprecation_compatibility():
         cfg.CONF.set_default('stats_update_threads',
                              cfg.CONF.health_manager.status_update_threads,
                              group='health_manager')
+
+
+def setup_remote_pydev_debug():
+    """Required setup for remote debugging."""
+
+    pydev_debug_host = os.environ.get('PYDEV_DEBUG_HOST')
+    pydev_debug_port = os.environ.get('PYDEV_DEBUG_PORT')
+
+    if not pydev_debug_host or not pydev_debug_port:
+        return
+
+    try:
+        try:
+            from pydev import pydevd
+        except ImportError:
+            import pydevd
+
+        LOG.warning("Connecting to remote debugger. Once connected, resume "
+                    "the program on the debugger to continue with the "
+                    "initialization of the service.")
+        pydevd.settrace(pydev_debug_host,
+                        port=int(pydev_debug_port),
+                        stdoutToServer=True,
+                        stderrToServer=True)
+    except Exception:
+        LOG.exception('Unable to join debugger, please make sure that the '
+                      'debugger processes is listening on debug-host '
+                      '\'%(debug-host)s\' debug-port \'%(debug-port)s\'.',
+                      {'debug-host': pydev_debug_host,
+                       'debug-port': pydev_debug_port})
+        raise
