@@ -193,11 +193,14 @@ HEALTH_MONITOR_DEFAULT_URL_PATH = '/'
 TYPE = 'type'
 URL_PATH = 'url_path'
 HTTP_METHOD = 'http_method'
+HTTP_VERSION = 'http_version'
 EXPECTED_CODES = 'expected_codes'
 DELAY = 'delay'
 TIMEOUT = 'timeout'
 MAX_RETRIES = 'max_retries'
+MAX_RETRIES_DOWN = 'max_retries_down'
 RISE_THRESHOLD = 'rise_threshold'
+DOMAIN_NAME = 'domain_name'
 
 UPDATE_STATS = 'UPDATE_STATS'
 UPDATE_HEALTH = 'UPDATE_HEALTH'
@@ -209,9 +212,11 @@ MAX_PORT_NUMBER = 65535
 DEFAULT_CONNECTION_LIMIT = -1
 MIN_CONNECTION_LIMIT = -1
 
+DEFAULT_WEIGHT = 1
 MIN_WEIGHT = 0
 MAX_WEIGHT = 256
 
+DEFAULT_MAX_RETRIES_DOWN = 3
 MIN_HM_RETRIES = 1
 MAX_HM_RETRIES = 10
 
@@ -340,6 +345,8 @@ BYTES_IN = 'bytes_in'
 BYTES_OUT = 'bytes_out'
 REQUEST_ERRORS = 'request_errors'
 TOTAL_CONNECTIONS = 'total_connections'
+NAME = 'name'
+PROVIDER_NAME = 'provider_name'
 
 CERT_ROTATE_AMPHORA_FLOW = 'octavia-cert-rotate-amphora-flow'
 CREATE_AMPHORA_FLOW = 'octavia-create-amphora-flow'
@@ -551,6 +558,7 @@ SYSCTL_CMD = '/sbin/sysctl'
 AMP_ACTION_START = 'start'
 AMP_ACTION_STOP = 'stop'
 AMP_ACTION_RELOAD = 'reload'
+AMP_ACTION_RESTART = 'restart'
 GLANCE_IMAGE_ACTIVE = 'active'
 
 INIT_SYSTEMD = 'systemd'
