@@ -364,6 +364,10 @@ class AmphoraFlows(object):
             rebind={constants.AMPHORA: constants.FAILED_AMPHORA},
             requires=constants.AMPHORA))
 
+        failover_amphora_flow.add(network_tasks.FailoverPreparationForAmphora(
+            rebind={constants.AMPHORA: constants.FAILED_AMPHORA},
+            requires=constants.AMPHORA))
+
         # Note: It seems intuitive to boot an amphora prior to deleting
         #       the old amphora, however this is a complicated issue.
         #       If the target host (due to anit-affinity) is resource
@@ -470,7 +474,7 @@ class AmphoraFlows(object):
             update_amps_subflow.add(
                 amphora_driver_tasks.AmpListenersUpdate(
                     name=constants.AMP_LISTENER_UPDATE + '-' + str(amp_index),
-                    requires=(constants.LISTENERS, constants.AMPHORAE),
+                    requires=(constants.LOADBALANCER, constants.AMPHORAE),
                     inject={constants.AMPHORA_INDEX: amp_index,
                             constants.TIMEOUT_DICT: timeout_dict}))
             amp_index += 1
@@ -514,8 +518,7 @@ class AmphoraFlows(object):
                     requires=constants.AMPHORA))
 
         failover_amphora_flow.add(amphora_driver_tasks.ListenersStart(
-            requires=(constants.LOADBALANCER, constants.LISTENERS,
-                      constants.AMPHORA)))
+            requires=(constants.LOADBALANCER, constants.AMPHORA)))
         failover_amphora_flow.add(
             database_tasks.DisableAmphoraHealthMonitoring(
                 rebind={constants.AMPHORA: constants.FAILED_AMPHORA},
