@@ -13,15 +13,15 @@
 # under the License.
 #
 
-import logging
-
 from oslo_config import cfg
+from oslo_log import log as logging
 import six
 from taskflow import task
 from taskflow.types import failure
 
 from octavia.common import constants
 from octavia.common import utils
+from octavia.controller.worker import task_utils
 from octavia.network import base
 from octavia.network import data_models as n_data_models
 
@@ -34,8 +34,14 @@ class BaseNetworkTask(task.Task):
 
     def __init__(self, **kwargs):
         super(BaseNetworkTask, self).__init__(**kwargs)
+        self._network_driver = None
+        self.task_utils = task_utils.TaskUtils()
 
-        self.network_driver = utils.get_network_driver()
+    @property
+    def network_driver(self):
+        if self._network_driver is None:
+            self._network_driver = utils.get_network_driver()
+        return self._network_driver
 
 
 class CalculateAmphoraDelta(BaseNetworkTask):
@@ -138,7 +144,7 @@ class PlugNetworks(BaseNetworkTask):
 
         if not delta:
             LOG.debug("No network deltas for amphora id: %s", amphora.id)
-            return None
+            return
 
         # add nics
         for nic in delta.add_nics:
@@ -173,7 +179,7 @@ class UnPlugNetworks(BaseNetworkTask):
         LOG.debug("Unplug network for amphora")
         if not delta:
             LOG.debug("No network deltas for amphora id: %s", amphora.id)
-            return None
+            return
 
         for nic in delta.delete_nics:
             try:
@@ -181,10 +187,9 @@ class UnPlugNetworks(BaseNetworkTask):
                                                    nic.network_id)
             except base.NetworkNotFound:
                 LOG.debug("Network %d not found", nic.network_id)
-                pass
             except Exception:
                 LOG.exception("Unable to unplug network")
-                pass  # Todo(german) follow up if that makes sense
+                # TODO(xgerman) follow up if that makes sense
 
 
 class GetMemberPorts(BaseNetworkTask):
@@ -210,6 +215,55 @@ class GetMemberPorts(BaseNetworkTask):
         return member_ports
 
 
+class HandleNetworkDelta(BaseNetworkTask):
+    """Task to plug and unplug networks
+
+    Plug or unplug networks based on delta
+    """
+
+    def execute(self, amphora, delta):
+        """Handle network plugging based off deltas."""
+        added_ports = {}
+        added_ports[amphora.id] = []
+        for nic in delta.add_nics:
+            interface = self.network_driver.plug_network(delta.compute_id,
+                                                         nic.network_id)
+            port = self.network_driver.get_port(interface.port_id)
+            port.network = self.network_driver.get_network(port.network_id)
+            for fixed_ip in port.fixed_ips:
+                fixed_ip.subnet = self.network_driver.get_subnet(
+                    fixed_ip.subnet_id)
+            added_ports[amphora.id].append(port)
+        for nic in delta.delete_nics:
+            try:
+                self.network_driver.unplug_network(delta.compute_id,
+                                                   nic.network_id)
+            except base.NetworkNotFound:
+                LOG.debug("Network %d not found ", nic.network_id)
+            except Exception:
+                LOG.exception("Unable to unplug network")
+        return added_ports
+
+    def revert(self, result, amphora, delta, *args, **kwargs):
+        """Handle a network plug or unplug failures."""
+
+        if isinstance(result, failure.Failure):
+            return
+
+        if not delta:
+            return
+
+        LOG.warning("Unable to plug networks for amp id %s",
+                    delta.amphora_id)
+
+        for nic in delta.add_nics:
+            try:
+                self.network_driver.unplug_network(delta.compute_id,
+                                                   nic.network_id)
+            except Exception:
+                pass
+
+
 class HandleNetworkDeltas(BaseNetworkTask):
     """Task to plug and unplug networks
 
@@ -284,6 +338,7 @@ class PlugVIP(BaseNetworkTask):
             # Make sure we have the current port IDs for cleanup
             for amp_data in result:
                 for amphora in six.moves.filter(
+                        # pylint: disable=cell-var-from-loop
                         lambda amp: amp.id == amp_data.id,
                         loadbalancer.amphorae):
                     amphora.vrrp_port_id = amp_data.vrrp_port_id
@@ -296,6 +351,60 @@ class PlugVIP(BaseNetworkTask):
                       {'vip': loadbalancer.vip.ip_address, 'except': e})
 
 
+class UpdateVIPSecurityGroup(BaseNetworkTask):
+    """Task to setup SG for LB."""
+
+    def execute(self, loadbalancer):
+        """Task to setup SG for LB."""
+
+        LOG.debug("Setup SG for loadbalancer id: %s", loadbalancer.id)
+
+        self.network_driver.update_vip_sg(loadbalancer, loadbalancer.vip)
+
+
+class GetSubnetFromVIP(BaseNetworkTask):
+    """Task to plumb a VIP."""
+
+    def execute(self, loadbalancer):
+        """Plumb a vip to an amphora."""
+
+        LOG.debug("Getting subnet for LB: %s", loadbalancer.id)
+
+        return self.network_driver.get_subnet(loadbalancer.vip.subnet_id)
+
+
+class PlugVIPAmpphora(BaseNetworkTask):
+    """Task to plumb a VIP."""
+
+    def execute(self, loadbalancer, amphora, subnet):
+        """Plumb a vip to an amphora."""
+
+        LOG.debug("Plumbing VIP for amphora id: %s", amphora.id)
+
+        amp_data = self.network_driver.plug_aap_port(
+            loadbalancer, loadbalancer.vip, amphora, subnet)
+        return amp_data
+
+    def revert(self, result, loadbalancer, amphora, subnet, *args, **kwargs):
+        """Handle a failure to plumb a vip."""
+
+        if isinstance(result, failure.Failure):
+            return
+        LOG.warning("Unable to plug VIP for amphora id %s "
+                    "load balancer id %s",
+                    amphora.id, loadbalancer.id)
+
+        try:
+            amphora.vrrp_port_id = result.vrrp_port_id
+            amphora.ha_port_id = result.ha_port_id
+
+            self.network_driver.unplug_aap_port(loadbalancer.vip,
+                                                amphora, subnet)
+        except Exception as e:
+            LOG.error('Failed to unplug AAP port. Resources may still be in '
+                      'use for VIP: %s due to error: %s', loadbalancer.vip, e)
+
+
 class UnplugVIP(BaseNetworkTask):
     """Task to unplug the vip."""
 
@@ -366,6 +475,25 @@ class UpdateVIP(BaseNetworkTask):
         self.network_driver.update_vip(loadbalancer)
 
 
+class UpdateVIPForDelete(BaseNetworkTask):
+    """Task to update a VIP for listener delete flows."""
+
+    def execute(self, loadbalancer):
+        LOG.debug("Updating VIP for listener delete on load_balancer %s.",
+                  loadbalancer.id)
+
+        self.network_driver.update_vip(loadbalancer, for_delete=True)
+
+
+class GetAmphoraNetworkConfigs(BaseNetworkTask):
+    """Task to retrieve amphora network details."""
+
+    def execute(self, loadbalancer, amphora=None):
+        LOG.debug("Retrieving vip network details.")
+        return self.network_driver.get_network_configs(loadbalancer,
+                                                       amphora=amphora)
+
+
 class GetAmphoraeNetworkConfigs(BaseNetworkTask):
     """Task to retrieve amphorae network details."""
 
@@ -447,3 +575,88 @@ class WaitForPortDetach(BaseNetworkTask):
         LOG.debug('Waiting for ports to detach from amphora: %(amp_id)s.',
                   {'amp_id': amphora.id})
         self.network_driver.wait_for_port_detach(amphora)
+
+
+class ApplyQos(BaseNetworkTask):
+    """Apply Quality of Services to the VIP"""
+
+    def _apply_qos_on_vrrp_ports(self, loadbalancer, amps_data, qos_policy_id,
+                                 is_revert=False, request_qos_id=None):
+        """Call network driver to apply QoS Policy on the vrrp ports."""
+        if not amps_data:
+            amps_data = loadbalancer.amphorae
+
+        apply_qos = ApplyQosAmphora()
+        for amp_data in amps_data:
+            apply_qos._apply_qos_on_vrrp_port(loadbalancer, amp_data,
+                                              qos_policy_id)
+
+    def execute(self, loadbalancer, amps_data=None, update_dict=None):
+        """Apply qos policy on the vrrp ports which are related with vip."""
+        qos_policy_id = loadbalancer.vip.qos_policy_id
+        if not qos_policy_id and (
+            update_dict and (
+                'vip' not in update_dict or
+                'qos_policy_id' not in update_dict['vip'])):
+            return
+        self._apply_qos_on_vrrp_ports(loadbalancer, amps_data, qos_policy_id)
+
+    def revert(self, result, loadbalancer, amps_data=None, update_dict=None,
+               *args, **kwargs):
+        """Handle a failure to apply QoS to VIP"""
+        request_qos_id = loadbalancer.vip.qos_policy_id
+        orig_lb = self.task_utils.get_current_loadbalancer_from_db(
+            loadbalancer.id)
+        orig_qos_id = orig_lb.vip.qos_policy_id
+        if request_qos_id != orig_qos_id:
+            self._apply_qos_on_vrrp_ports(loadbalancer, amps_data, orig_qos_id,
+                                          is_revert=True,
+                                          request_qos_id=request_qos_id)
+        return
+
+
+class ApplyQosAmphora(BaseNetworkTask):
+    """Apply Quality of Services to the VIP"""
+
+    def _apply_qos_on_vrrp_port(self, loadbalancer, amp_data, qos_policy_id,
+                                is_revert=False, request_qos_id=None):
+        """Call network driver to apply QoS Policy on the vrrp ports."""
+        try:
+            self.network_driver.apply_qos_on_port(qos_policy_id,
+                                                  amp_data.vrrp_port_id)
+        except Exception:
+            if not is_revert:
+                raise
+            else:
+                LOG.warning('Failed to undo qos policy %(qos_id)s '
+                            'on vrrp port: %(port)s from '
+                            'amphorae: %(amp)s',
+                            {'qos_id': request_qos_id,
+                             'port': amp_data.vrrp_port_id,
+                             'amp': [amp.id for amp in amp_data]})
+
+    def execute(self, loadbalancer, amp_data=None, update_dict=None):
+        """Apply qos policy on the vrrp ports which are related with vip."""
+        qos_policy_id = loadbalancer.vip.qos_policy_id
+        if not qos_policy_id and (
+            update_dict and (
+                'vip' not in update_dict or
+                'qos_policy_id' not in update_dict['vip'])):
+            return
+        self._apply_qos_on_vrrp_port(loadbalancer, amp_data, qos_policy_id)
+
+    def revert(self, result, loadbalancer, amp_data=None, update_dict=None,
+               *args, **kwargs):
+        """Handle a failure to apply QoS to VIP"""
+        try:
+            request_qos_id = loadbalancer.vip.qos_policy_id
+            orig_lb = self.task_utils.get_current_loadbalancer_from_db(
+                loadbalancer.id)
+            orig_qos_id = orig_lb.vip.qos_policy_id
+            if request_qos_id != orig_qos_id:
+                self._apply_qos_on_vrrp_port(loadbalancer, amp_data,
+                                             orig_qos_id, is_revert=True,
+                                             request_qos_id=request_qos_id)
+        except Exception as e:
+            LOG.error('Failed to remove QoS policy: %s from port: %s due '
+                      'to error: %s', orig_qos_id, amp_data.vrrp_port_id, e)
