@@ -1,5 +1,6 @@
 #    Copyright 2014 Rackspace
 #    Copyright 2016 Blue Box, an IBM Company
+#    Copyright 2017 Walmart Stores Inc.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -13,6 +14,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+
 from oslo_db.sqlalchemy import models
 import sqlalchemy as sa
 from sqlalchemy.ext import orderinglist
@@ -21,6 +23,8 @@ from sqlalchemy.orm import validates
 from sqlalchemy.sql import func
 
 from octavia.api.v2.types import amphora
+from octavia.api.v2.types import flavor_profile
+from octavia.api.v2.types import flavors
 from octavia.api.v2.types import health_monitor
 from octavia.api.v2.types import l7policy
 from octavia.api.v2.types import l7rule
@@ -29,6 +33,7 @@ from octavia.api.v2.types import load_balancer
 from octavia.api.v2.types import member
 from octavia.api.v2.types import pool
 from octavia.api.v2.types import quotas
+from octavia.common import constants
 from octavia.common import data_models
 from octavia.db import base_models
 from octavia.i18n import _
@@ -169,7 +174,8 @@ class ListenerStatistics(base_models.BASE):
 
 
 class Member(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
-             models.TimestampMixin, base_models.NameMixin):
+             models.TimestampMixin, base_models.NameMixin,
+             base_models.TagMixin):
 
     __data_model__ = data_models.Member
 
@@ -206,10 +212,18 @@ class Member(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
     enabled = sa.Column(sa.Boolean(), nullable=False)
     pool = orm.relationship("Pool", back_populates="members")
 
+    _tags = orm.relationship(
+        'Tags',
+        single_parent=True,
+        lazy='subquery',
+        cascade='all,delete-orphan',
+        primaryjoin='and_(foreign(Tags.resource_id)==Member.id)'
+    )
+
 
 class HealthMonitor(base_models.BASE, base_models.IdMixin,
                     base_models.ProjectMixin, models.TimestampMixin,
-                    base_models.NameMixin):
+                    base_models.NameMixin, base_models.TagMixin):
 
     __data_model__ = data_models.HealthMonitor
 
@@ -252,10 +266,19 @@ class HealthMonitor(base_models.BASE, base_models.IdMixin,
         sa.ForeignKey("operating_status.name",
                       name="fk_health_monitor_operating_status_name"),
         nullable=False)
+    _tags = orm.relationship(
+        'Tags',
+        single_parent=True,
+        lazy='subquery',
+        cascade='all,delete-orphan',
+        primaryjoin='and_(foreign(Tags.resource_id)==HealthMonitor.id)'
+    )
+    http_version = sa.Column(sa.Float, nullable=True)
+    domain_name = sa.Column(sa.String(255), nullable=True)
 
 
 class Pool(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
-           models.TimestampMixin, base_models.NameMixin):
+           models.TimestampMixin, base_models.NameMixin, base_models.TagMixin):
 
     __data_model__ = data_models.Pool
 
@@ -300,6 +323,17 @@ class Pool(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
                                           back_populates="default_pool")
     l7policies = orm.relationship("L7Policy", uselist=True,
                                   back_populates="redirect_pool")
+    _tags = orm.relationship(
+        'Tags',
+        single_parent=True,
+        lazy='subquery',
+        cascade='all,delete-orphan',
+        primaryjoin='and_(foreign(Tags.resource_id)==Pool.id)'
+    )
+    tls_certificate_id = sa.Column(sa.String(255), nullable=True)
+    ca_tls_certificate_id = sa.Column(sa.String(255), nullable=True)
+    crl_container_id = sa.Column(sa.String(255), nullable=True)
+    tls_enabled = sa.Column(sa.Boolean, default=False, nullable=False)
 
     # This property should be a unique list of any listeners that reference
     # this pool as its default_pool and any listeners referenced by enabled
@@ -321,7 +355,7 @@ class Pool(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
 
 class LoadBalancer(base_models.BASE, base_models.IdMixin,
                    base_models.ProjectMixin, models.TimestampMixin,
-                   base_models.NameMixin):
+                   base_models.NameMixin, base_models.TagMixin):
 
     __data_model__ = data_models.LoadBalancer
 
@@ -355,6 +389,16 @@ class LoadBalancer(base_models.BASE, base_models.IdMixin,
                              back_populates="load_balancer")
     listeners = orm.relationship('Listener', cascade='delete', uselist=True,
                                  back_populates='load_balancer')
+    _tags = orm.relationship(
+        'Tags',
+        single_parent=True,
+        lazy='subquery',
+        cascade='all,delete-orphan',
+        primaryjoin='and_(foreign(Tags.resource_id)==LoadBalancer.id)'
+    )
+    flavor_id = sa.Column(
+        sa.String(36),
+        sa.ForeignKey("flavor.id", name="fk_lb_flavor_id"), nullable=True)
 
 
 class VRRPGroup(base_models.BASE):
@@ -402,7 +446,7 @@ class Vip(base_models.BASE):
 
 class Listener(base_models.BASE, base_models.IdMixin,
                base_models.ProjectMixin, models.TimestampMixin,
-               base_models.NameMixin):
+               base_models.NameMixin, base_models.TagMixin):
 
     __data_model__ = data_models.Listener
 
@@ -426,7 +470,7 @@ class Listener(base_models.BASE, base_models.IdMixin,
         sa.String(36),
         sa.ForeignKey("load_balancer.id", name="fk_listener_load_balancer_id"),
         nullable=True)
-    tls_certificate_id = sa.Column(sa.String(36), nullable=True)
+    tls_certificate_id = sa.Column(sa.String(255), nullable=True)
     default_pool_id = sa.Column(
         sa.String(36),
         sa.ForeignKey("pool.id", name="fk_listener_pool_id"),
@@ -461,6 +505,21 @@ class Listener(base_models.BASE, base_models.IdMixin,
     timeout_member_connect = sa.Column(sa.Integer, nullable=True)
     timeout_member_data = sa.Column(sa.Integer, nullable=True)
     timeout_tcp_inspect = sa.Column(sa.Integer, nullable=True)
+    client_ca_tls_certificate_id = sa.Column(sa.String(255), nullable=True)
+    client_authentication = sa.Column(
+        sa.String(10),
+        sa.ForeignKey("client_authentication_mode.name",
+                      name="fk_listener_client_authentication_mode_name"),
+        nullable=False, default=constants.CLIENT_AUTH_NONE)
+    client_crl_container_id = sa.Column(sa.String(255), nullable=True)
+
+    _tags = orm.relationship(
+        'Tags',
+        single_parent=True,
+        lazy='subquery',
+        cascade='all,delete-orphan',
+        primaryjoin='and_(foreign(Tags.resource_id)==Listener.id)'
+    )
 
     # This property should be a unique list of the default_pool and anything
     # referenced by enabled L7Policies with at least one rule that also
@@ -491,7 +550,6 @@ class SNI(base_models.BASE):
     __table_args__ = (
         sa.PrimaryKeyConstraint('listener_id', 'tls_container_id'),
     )
-
     listener_id = sa.Column(
         sa.String(36),
         sa.ForeignKey("listener.id", name="fk_sni_listener_id"),
@@ -537,6 +595,7 @@ class Amphora(base_models.BASE, base_models.IdMixin, models.TimestampMixin):
     image_id = sa.Column(sa.String(36), nullable=True)
     load_balancer = orm.relationship("LoadBalancer", uselist=False,
                                      back_populates='amphorae')
+    compute_flavor = sa.Column(sa.String(255), nullable=True)
 
 
 class AmphoraHealth(base_models.BASE):
@@ -552,7 +611,7 @@ class AmphoraHealth(base_models.BASE):
 
 
 class L7Rule(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
-             models.TimestampMixin):
+             models.TimestampMixin, base_models.TagMixin):
 
     __data_model__ = data_models.L7Rule
 
@@ -592,10 +651,18 @@ class L7Rule(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
         sa.ForeignKey("operating_status.name",
                       name="fk_l7rule_operating_status_name"),
         nullable=False)
+    _tags = orm.relationship(
+        'Tags',
+        single_parent=True,
+        lazy='subquery',
+        cascade='all,delete-orphan',
+        primaryjoin='and_(foreign(Tags.resource_id)==L7Rule.id)'
+    )
 
 
 class L7Policy(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
-               models.TimestampMixin, base_models.NameMixin):
+               models.TimestampMixin, base_models.NameMixin,
+               base_models.TagMixin):
 
     __data_model__ = data_models.L7Policy
 
@@ -621,6 +688,10 @@ class L7Policy(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
     redirect_url = sa.Column(
         sa.String(255),
         nullable=True)
+    redirect_prefix = sa.Column(
+        sa.String(255),
+        nullable=True)
+    redirect_http_code = sa.Column(sa.Integer, nullable=True)
     position = sa.Column(sa.Integer, nullable=False)
     enabled = sa.Column(sa.Boolean(), nullable=False)
     listener = orm.relationship("Listener", uselist=False,
@@ -639,6 +710,13 @@ class L7Policy(base_models.BASE, base_models.IdMixin, base_models.ProjectMixin,
         sa.ForeignKey("operating_status.name",
                       name="fk_l7policy_operating_status_name"),
         nullable=False)
+    _tags = orm.relationship(
+        'Tags',
+        single_parent=True,
+        lazy='subquery',
+        cascade='all,delete-orphan',
+        primaryjoin='and_(foreign(Tags.resource_id)==L7Policy.id)'
+    )
 
 
 class Quotas(base_models.BASE):
@@ -660,3 +738,54 @@ class Quotas(base_models.BASE):
     in_use_load_balancer = sa.Column(sa.Integer(), nullable=True)
     in_use_member = sa.Column(sa.Integer(), nullable=True)
     in_use_pool = sa.Column(sa.Integer(), nullable=True)
+
+
+class FlavorProfile(base_models.BASE, base_models.IdMixin,
+                    base_models.NameMixin):
+
+    __data_model__ = data_models.FlavorProfile
+
+    __tablename__ = "flavor_profile"
+
+    __v2_wsme__ = flavor_profile.FlavorProfileResponse
+
+    provider_name = sa.Column(sa.String(255), nullable=False)
+    flavor_data = sa.Column(sa.String(4096), nullable=False)
+
+
+class Flavor(base_models.BASE,
+             base_models.IdMixin,
+             base_models.NameMixin):
+
+    __data_model__ = data_models.Flavor
+
+    __tablename__ = "flavor"
+
+    __v2_wsme__ = flavors.FlavorResponse
+
+    __table_args__ = (
+        sa.UniqueConstraint('name',
+                            name='uq_flavor_name'),
+    )
+
+    description = sa.Column(sa.String(255), nullable=True)
+    enabled = sa.Column(sa.Boolean(), nullable=False)
+    flavor_profile_id = sa.Column(
+        sa.String(36),
+        sa.ForeignKey("flavor_profile.id",
+                      name="fk_flavor_flavor_profile_id"),
+        nullable=False)
+
+
+class ClientAuthenticationMode(base_models.BASE):
+
+    __tablename__ = "client_authentication_mode"
+
+    name = sa.Column(sa.String(10), primary_key=True, nullable=False)
+
+
+class SparesPool(base_models.BASE):
+
+    __tablename__ = "spares_pool"
+
+    updated_at = sa.Column(sa.DateTime, primary_key=True, nullable=True)
