@@ -196,7 +196,7 @@ class BaseRepository(object):
         if hasattr(self.model_class, 'status'):
             query = query.filter_by(status=consts.DELETED)
         else:
-            query = query.filter_by(operating_status=consts.DELETED)
+            query = query.filter_by(provisioning_status=consts.DELETED)
         # Do not load any relationship
         query = query.options(noload('*'))
         model_list = query.all()
@@ -1137,6 +1137,7 @@ class AmphoraRepository(BaseRepository):
                 id=amphora_id).first()
             load_balancer.amphorae.append(amphora)
 
+    @oslo_db_api.wrap_db_retry(max_retries=5, retry_on_deadlock=True)
     def allocate_and_associate(self, session, load_balancer_id):
         """Allocate an amphora for a load balancer.
 
@@ -1252,6 +1253,7 @@ class AmphoraRepository(BaseRepository):
             "load_balancer.operating_status AS lb_op_status, "
             "listener.id AS list_id, "
             "listener.operating_status AS list_op_status, "
+            "listener.enabled AS list_enabled, "
             "listener.protocol AS list_protocol, "
             "pool.id AS pool_id, "
             "pool.operating_status AS pool_op_status, "
@@ -1277,7 +1279,8 @@ class AmphoraRepository(BaseRepository):
                 lb['operating_status'] = row['lb_op_status']
             if row['list_id'] and row['list_id'] not in listeners:
                 listener = {'operating_status': row['list_op_status'],
-                            'protocol': row['list_protocol']}
+                            'protocol': row['list_protocol'],
+                            'enabled': row['list_enabled']}
                 listeners[row['list_id']] = listener
             if row['pool_id']:
                 if row['pool_id'] in pools and row['member_id']:
@@ -1530,6 +1533,10 @@ class L7RuleRepository(BaseRepository):
         with session.begin(subtransactions=True):
             if not model_kwargs.get('id'):
                 model_kwargs.update(id=uuidutils.generate_uuid())
+            if model_kwargs.get('l7policy_id'):
+                l7policy_db = session.query(models.L7Policy).filter_by(
+                    id=model_kwargs.get('l7policy_id')).first()
+                model_kwargs.update(l7policy=l7policy_db)
             l7rule = self.model_class(**model_kwargs)
             validate.l7rule_data(l7rule)
             session.add(l7rule)
@@ -1678,6 +1685,10 @@ class L7PolicyRepository(BaseRepository):
                 pool_db = session.query(models.Pool).filter_by(
                     id=model_kwargs.get('redirect_pool_id')).first()
                 model_kwargs.update(redirect_pool=pool_db)
+            if model_kwargs.get('listener_id'):
+                listener_db = session.query(models.Listener).filter_by(
+                    id=model_kwargs.get('listener_id')).first()
+                model_kwargs.update(listener=listener_db)
             l7policy = self.model_class(
                 **validate.sanitize_l7policy_api_args(model_kwargs,
                                                       create=True))
