@@ -17,6 +17,7 @@ import operator
 from keystoneauth1 import loading as ks_loading
 from oslo_config import cfg
 
+import octavia.certificates.common.local
 import octavia.common.config
 from octavia.common import constants
 
@@ -33,7 +34,9 @@ def list_opts():
         ('health_manager', octavia.common.config.healthmanager_opts),
         ('controller_worker', octavia.common.config.controller_worker_opts),
         ('task_flow', octavia.common.config.task_flow_opts),
-        ('certificates', octavia.common.config.certificate_opts),
+        ('certificates', itertools.chain(
+            octavia.common.config.certificate_opts,
+            octavia.certificates.common.local.certgen_opts)),
         ('house_keeping', octavia.common.config.house_keeping_opts),
         ('keepalived_vrrp', octavia.common.config.keepalived_vrrp_opts),
         ('anchor', octavia.common.config.anchor_opts),
@@ -41,6 +44,8 @@ def list_opts():
         ('neutron', octavia.common.config.neutron_opts),
         ('glance', octavia.common.config.glance_opts),
         ('quotas', octavia.common.config.quota_opts),
+        ('audit', octavia.common.config.audit_opts),
+        ('driver_agent', octavia.common.config.driver_agent_opts),
         add_auth_opts(),
     ]
 
