@@ -16,6 +16,8 @@ import copy
 
 import mock
 
+from octavia_lib.api.drivers import exceptions as lib_exceptions
+
 from octavia.api.drivers import data_models as driver_dm
 from octavia.api.drivers import exceptions as driver_exceptions
 from octavia.api.drivers import utils
@@ -39,24 +41,41 @@ class TestUtils(base.TestCase):
                             "arg1", foo="arg2")
         mock_driver_method.assert_called_with("arg1", foo="arg2")
 
-        # Test driver raising DriverError
+        # Test driver raising different types of DriverError
         mock_driver_method.side_effect = driver_exceptions.DriverError
+        self.assertRaises(exceptions.ProviderDriverError,
+                          utils.call_provider, "provider_name",
+                          mock_driver_method)
+        mock_driver_method.side_effect = lib_exceptions.DriverError
         self.assertRaises(exceptions.ProviderDriverError,
                           utils.call_provider, "provider_name",
                           mock_driver_method)
 
-        # Test driver raising NotImplementedError
+        # Test driver raising different types of NotImplementedError
         mock_driver_method.side_effect = driver_exceptions.NotImplementedError
+        self.assertRaises(exceptions.ProviderNotImplementedError,
+                          utils.call_provider, "provider_name",
+                          mock_driver_method)
+        mock_driver_method.side_effect = NotImplementedError
+        self.assertRaises(exceptions.ProviderNotImplementedError,
+                          utils.call_provider, "provider_name",
+                          mock_driver_method)
+        mock_driver_method.side_effect = lib_exceptions.NotImplementedError
         self.assertRaises(exceptions.ProviderNotImplementedError,
                           utils.call_provider, "provider_name",
                           mock_driver_method)
 
-        # Test driver raising UnsupportedOptionError
+        # Test driver raising different types of UnsupportedOptionError
         mock_driver_method.side_effect = (
             driver_exceptions.UnsupportedOptionError)
         self.assertRaises(exceptions.ProviderUnsupportedOptionError,
                           utils.call_provider, "provider_name",
                           mock_driver_method)
+        mock_driver_method.side_effect = (
+            lib_exceptions.UnsupportedOptionError)
+        self.assertRaises(exceptions.ProviderUnsupportedOptionError,
+                          utils.call_provider, "provider_name",
+                          mock_driver_method)
 
         # Test driver raising DriverError
         mock_driver_method.side_effect = Exception
@@ -84,13 +103,23 @@ class TestUtils(base.TestCase):
         self.assertEqual({'admin_state_up': True},
                          result_dict)
 
+    @mock.patch('octavia.api.drivers.utils._get_secret_data')
     @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
-    def test_lb_dict_to_provider_dict(self, mock_load_cert):
+    def test_lb_dict_to_provider_dict(self, mock_load_cert, mock_secret):
         cert1 = data_models.TLSContainer(certificate='cert 1')
         cert2 = data_models.TLSContainer(certificate='cert 2')
         cert3 = data_models.TLSContainer(certificate='cert 3')
-        mock_load_cert.return_value = {'tls_cert': cert1,
-                                       'sni_certs': [cert2, cert3]}
+        mock_secret.side_effect = ['X509 POOL CA CERT FILE',
+                                   'X509 POOL CRL FILE', 'ca cert',
+                                   'X509 CRL FILE', 'ca cert', 'X509 CRL FILE',
+                                   'X509 POOL CA CERT FILE',
+                                   'X509 CRL FILE']
+        listener_certs = {'tls_cert': cert1, 'sni_certs': [cert2, cert3]}
+        pool_cert = data_models.TLSContainer(certificate='pool cert')
+        pool_certs = {'tls_cert': pool_cert, 'sni_certs': []}
+        mock_load_cert.side_effect = [pool_certs, listener_certs,
+                                      listener_certs, listener_certs,
+                                      listener_certs]
         test_lb_dict = {'name': 'lb1',
                         'project_id': self.sample_data.project_id,
                         'vip_subnet_id': self.sample_data.subnet_id,
@@ -114,7 +143,6 @@ class TestUtils(base.TestCase):
             'listeners': self.sample_data.provider_listeners,
             'description': '',
             'project_id': self.sample_data.project_id,
-            'flavor_id': '',
             'vip_port_id': self.sample_data.port_id,
             'vip_qos_policy_id': self.sample_data.qos_policy_id,
             'vip_network_id': self.sample_data.network_id,
@@ -157,8 +185,13 @@ class TestUtils(base.TestCase):
         self.assertEqual(ref_provider_list.to_dict(render_unsets=True),
                          provider_list.to_dict(render_unsets=True))
 
+    @mock.patch('octavia.api.drivers.utils._get_secret_data')
     @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
-    def test_db_listeners_to_provider_listeners(self, mock_load_cert):
+    def test_db_listeners_to_provider_listeners(self, mock_load_cert,
+                                                mock_secret):
+        mock_secret.side_effect = ['ca cert', 'X509 CRL FILE',
+                                   'ca cert', 'X509 CRL FILE',
+                                   'ca cert', 'X509 CRL FILE']
         cert1 = data_models.TLSContainer(certificate='cert 1')
         cert2 = data_models.TLSContainer(certificate='cert 2')
         cert3 = data_models.TLSContainer(certificate='cert 3')
@@ -169,46 +202,37 @@ class TestUtils(base.TestCase):
         self.assertEqual(self.sample_data.provider_listeners,
                          provider_listeners)
 
+    @mock.patch('octavia.api.drivers.utils._get_secret_data')
     @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
-    def test_listener_dict_to_provider_dict(self, mock_load_cert):
+    def test_listener_dict_to_provider_dict(self, mock_load_cert, mock_secret):
+        mock_secret.side_effect = ['ca cert', 'X509 CRL FILE',
+                                   'X509 POOL CA CERT FILE',
+                                   'X509 POOL CRL FILE']
         cert1 = data_models.TLSContainer(certificate='cert 1')
         cert2 = data_models.TLSContainer(certificate='cert 2')
         cert3 = data_models.TLSContainer(certificate='cert 3')
-        mock_load_cert.return_value = {'tls_cert': cert1,
-                                       'sni_certs': [cert2, cert3]}
-        provider_listener = utils.listener_dict_to_provider_dict(
-            self.sample_data.test_listener1_dict)
-        self.assertEqual(self.sample_data.provider_listener1_dict,
-                         provider_listener)
-
-    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
-    def test_listener_dict_to_provider_dict_load_cert_error(self,
-                                                            mock_load_cert):
-        mock_load_cert.side_effect = [exceptions.OctaviaException,
-                                      Exception]
-
-        # Test load_cert exception for_delete == False path
-        self.assertRaises(exceptions.OctaviaException,
-                          utils.listener_dict_to_provider_dict,
-                          self.sample_data.test_listener1_dict)
-
-    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
-    def test_listener_dict_to_provider_dict_load_cert_error_for_delete(
-            self, mock_load_cert):
-        mock_load_cert.side_effect = [Exception]
-
-        # Test load_cert exception for_delete == True path
+        listener_certs = {'tls_cert': cert1, 'sni_certs': [cert2, cert3]}
+        pool_cert = data_models.TLSContainer(certificate='pool cert')
+        pool_certs = {'tls_cert': pool_cert, 'sni_certs': []}
+        mock_load_cert.side_effect = [listener_certs, pool_certs]
+        # The reason to do this, as before the logic arrives the test func,
+        # there are two data sources, one is from db_dict, the other is from
+        # the api layer model_dict, actually, they are different and contain
+        # different fields. That's why the test_listener1_dict from sample data
+        # just contain the client_ca_tls_certificate_id for client certificate,
+        # not any other related fields. So we need to delete them.
         expect_prov = copy.deepcopy(self.sample_data.provider_listener1_dict)
         expect_pool_prov = copy.deepcopy(self.sample_data.provider_pool1_dict)
         expect_prov['default_pool'] = expect_pool_prov
-        del expect_prov['default_tls_container_data']
-        del expect_prov['sni_container_data']
         provider_listener = utils.listener_dict_to_provider_dict(
-            self.sample_data.test_listener1_dict, for_delete=True)
+            self.sample_data.test_listener1_dict)
         self.assertEqual(expect_prov, provider_listener)
 
+    @mock.patch('octavia.api.drivers.utils._get_secret_data')
     @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
-    def test_listener_dict_to_provider_dict_SNI(self, mock_load_cert):
+    def test_listener_dict_to_provider_dict_SNI(self, mock_load_cert,
+                                                mock_secret):
+        mock_secret.return_value = 'ca cert'
         cert1 = data_models.TLSContainer(certificate='cert 1')
         cert2 = data_models.TLSContainer(certificate='cert 2')
         cert3 = data_models.TLSContainer(certificate='cert 3')
@@ -221,27 +245,62 @@ class TestUtils(base.TestCase):
                           utils.listener_dict_to_provider_dict,
                           test_listener)
 
-    def test_db_pool_to_provider_pool(self):
+    @mock.patch('octavia.api.drivers.utils._get_secret_data')
+    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
+    def test_db_pool_to_provider_pool(self, mock_load_cert, mock_secret):
+        pool_cert = data_models.TLSContainer(certificate='pool cert')
+        mock_load_cert.return_value = {'tls_cert': pool_cert,
+                                       'sni_certs': None,
+                                       'client_ca_cert': None}
+        mock_secret.side_effect = ['X509 POOL CA CERT FILE',
+                                   'X509 POOL CRL FILE']
         provider_pool = utils.db_pool_to_provider_pool(
             self.sample_data.db_pool1)
         self.assertEqual(self.sample_data.provider_pool1, provider_pool)
 
-    def test_db_pool_to_provider_pool_partial(self):
+    @mock.patch('octavia.api.drivers.utils._get_secret_data')
+    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
+    def test_db_pool_to_provider_pool_partial(self, mock_load_cert,
+                                              mock_secret):
+        pool_cert = data_models.TLSContainer(certificate='pool cert')
+        mock_load_cert.return_value = {'tls_cert': pool_cert,
+                                       'sni_certs': None,
+                                       'client_ca_cert': None}
+        mock_secret.side_effect = ['X509 POOL CA CERT FILE',
+                                   'X509 POOL CRL FILE']
         test_db_pool = self.sample_data.db_pool1
         test_db_pool.members = [self.sample_data.db_member1]
         provider_pool = utils.db_pool_to_provider_pool(test_db_pool)
         self.assertEqual(self.sample_data.provider_pool1, provider_pool)
 
-    def test_db_pools_to_provider_pools(self):
+    @mock.patch('octavia.api.drivers.utils._get_secret_data')
+    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
+    def test_db_pools_to_provider_pools(self, mock_load_cert, mock_secret):
+        pool_cert = data_models.TLSContainer(certificate='pool cert')
+        mock_load_cert.return_value = {'tls_cert': pool_cert,
+                                       'sni_certs': None,
+                                       'client_ca_cert': None}
+        mock_secret.side_effect = ['X509 POOL CA CERT FILE',
+                                   'X509 POOL CRL FILE']
         provider_pools = utils.db_pools_to_provider_pools(
             self.sample_data.test_db_pools)
         self.assertEqual(self.sample_data.provider_pools, provider_pools)
 
-    def test_pool_dict_to_provider_dict(self):
+    @mock.patch('octavia.api.drivers.utils._get_secret_data')
+    @mock.patch('octavia.common.tls_utils.cert_parser.load_certificates_data')
+    def test_pool_dict_to_provider_dict(self, mock_load_cert, mock_secret):
+        pool_cert = data_models.TLSContainer(certificate='pool cert')
+        mock_load_cert.return_value = {'tls_cert': pool_cert,
+                                       'sni_certs': None,
+                                       'client_ca_cert': None}
+        mock_secret.side_effect = ['X509 POOL CA CERT FILE',
+                                   'X509 POOL CRL FILE']
+        expect_prov = copy.deepcopy(self.sample_data.provider_pool1_dict)
+        expect_prov.pop('crl_container_ref')
         provider_pool_dict = utils.pool_dict_to_provider_dict(
             self.sample_data.test_pool1_dict)
-        self.assertEqual(self.sample_data.provider_pool1_dict,
-                         provider_pool_dict)
+        provider_pool_dict.pop('crl_container_ref')
+        self.assertEqual(expect_prov, provider_pool_dict)
 
     def test_db_HM_to_provider_HM(self):
         provider_hm = utils.db_HM_to_provider_HM(self.sample_data.db_hm1)
@@ -252,6 +311,14 @@ class TestUtils(base.TestCase):
             self.sample_data.test_hm1_dict)
         self.assertEqual(self.sample_data.provider_hm1_dict, provider_hm_dict)
 
+    def test_HM_to_provider_HM_with_http_version_and_domain_name(self):
+        provider_hm = utils.db_HM_to_provider_HM(self.sample_data.db_hm2)
+        self.assertEqual(self.sample_data.provider_hm2, provider_hm)
+
+        provider_hm_dict = utils.hm_dict_to_provider_dict(
+            self.sample_data.test_hm2_dict)
+        self.assertEqual(self.sample_data.provider_hm2_dict, provider_hm_dict)
+
     def test_hm_dict_to_provider_dict_partial(self):
         provider_hm_dict = utils.hm_dict_to_provider_dict({'id': 1})
         self.assertEqual({'healthmonitor_id': 1}, provider_hm_dict)
