@@ -274,7 +274,7 @@ class TestValidations(base.TestCase):
         device_id = uuidutils.generate_uuid()
         port = network_models.Port(id=port_id, device_id=device_id)
         with mock.patch(
-                 'octavia.common.utils.get_network_driver') as net_mock:
+                'octavia.common.utils.get_network_driver') as net_mock:
             net_mock.return_value.get_port.device_id = port
             self.assertRaises(
                 exceptions.ValidationException,
@@ -374,6 +374,18 @@ class TestValidations(base.TestCase):
                               validate.qos_policy_exists,
                               qos_policy_id)
 
+    def test_qos_extension_enabled(self):
+        network_driver = mock.Mock()
+        network_driver.qos_enabled.return_value = True
+        self.assertIsNone(validate.qos_extension_enabled(network_driver))
+
+    def test_qos_extension_disabled(self):
+        network_driver = mock.Mock()
+        network_driver.qos_enabled.return_value = False
+        self.assertRaises(exceptions.ValidationException,
+                          validate.qos_extension_enabled,
+                          network_driver)
+
     def test_check_session_persistence(self):
         valid_cookie_name_dict = {'type': 'APP_COOKIE',
                                   'cookie_name': 'chocolate_chip'}
@@ -442,3 +454,13 @@ class TestValidations(base.TestCase):
         self.assertRaises(exceptions.InvalidOption,
                           validate.ip_not_reserved,
                           '2001:0DB8::5')
+
+    def test_is_flavor_spares_compatible(self):
+        not_compat_flavor = {constants.COMPUTE_FLAVOR: 'chocolate'}
+        compat_flavor = {constants.LOADBALANCER_TOPOLOGY:
+                         constants.TOPOLOGY_SINGLE}
+
+        self.assertTrue(validate.is_flavor_spares_compatible(None))
+        self.assertTrue(validate.is_flavor_spares_compatible(compat_flavor))
+        self.assertFalse(
+            validate.is_flavor_spares_compatible(not_compat_flavor))
