@@ -22,17 +22,17 @@ import octavia.common.exceptions as exceptions
 import octavia.common.tls_utils.cert_parser as cert_parser
 from octavia.tests.unit import base
 from octavia.tests.unit.common.sample_configs import sample_certs
-from octavia.tests.unit.common.sample_configs import sample_configs
+from octavia.tests.unit.common.sample_configs import sample_configs_combined
 
 
 class TestTLSParseUtils(base.TestCase):
     def test_alt_subject_name_parses(self):
         hosts = cert_parser.get_host_names(sample_certs.ALT_EXT_CRT)
         self.assertIn('www.cnfromsubject.org', hosts['cn'])
-        self.assertIn('www.hostfromdnsname1.com', hosts['dns_names'])
-        self.assertIn('www.hostfromdnsname2.com', hosts['dns_names'])
-        self.assertIn('www.hostfromdnsname3.com', hosts['dns_names'])
-        self.assertIn('www.hostfromdnsname4.com', hosts['dns_names'])
+        self.assertIn('www.hostFromDNSName1.com', hosts['dns_names'])
+        self.assertIn('www.hostFromDNSName2.com', hosts['dns_names'])
+        self.assertIn('www.hostFromDNSName3.com', hosts['dns_names'])
+        self.assertIn('www.hostFromDNSName4.com', hosts['dns_names'])
 
     def test_x509_parses(self):
         self.assertRaises(exceptions.UnreadableCert,
@@ -78,14 +78,25 @@ class TestTLSParseUtils(base.TestCase):
         self.assertTrue(
             cert_parser.validate_cert(
                 sample_certs.X509_CERT,
-                private_key=sample_certs.X509_CERT_KEY,
-                intermediates=(sample_certs.TEST_X509_IMDS +
-                               b"\nParser should ignore junk\n")))
+                private_key=sample_certs.X509_CERT_KEY.decode('utf-8')))
         self.assertRaises(exceptions.MisMatchedKey,
                           cert_parser.validate_cert,
                           sample_certs.X509_CERT,
                           private_key=sample_certs.X509_CERT_KEY_2)
 
+    def test_validate_cert_handles_intermediates(self):
+        self.assertTrue(
+            cert_parser.validate_cert(
+                sample_certs.X509_CERT,
+                private_key=sample_certs.X509_CERT_KEY,
+                intermediates=(sample_certs.X509_IMDS +
+                               b"\nParser should ignore junk\n")))
+        self.assertTrue(
+            cert_parser.validate_cert(
+                sample_certs.X509_CERT,
+                private_key=sample_certs.X509_CERT_KEY,
+                intermediates=sample_certs.X509_IMDS_LIST))
+
     def test_split_x509s(self):
         imds = []
         for x509Pem in cert_parser._split_x509s(sample_certs.TEST_X509_IMDS):
@@ -131,27 +142,45 @@ class TestTLSParseUtils(base.TestCase):
             exceptions.UnreadableCert,
             cert_parser._get_x509_from_der_bytes, b'bad data')
 
-    def test_load_certificates(self):
-        listener = sample_configs.sample_listener_tuple(tls=True, sni=True)
+    @mock.patch('oslo_context.context.RequestContext')
+    def test_load_certificates(self, mock_oslo):
+        listener = sample_configs_combined.sample_listener_tuple(
+            tls=True, sni=True, client_ca_cert=True)
         client = mock.MagicMock()
+        context = mock.Mock()
+        context.project_id = '12345'
         with mock.patch.object(cert_parser,
                                'get_host_names') as cp:
             with mock.patch.object(cert_parser,
                                    '_map_cert_tls_container'):
                 cp.return_value = {'cn': 'fakeCN'}
-                cert_parser.load_certificates_data(client, listener)
+                cert_parser.load_certificates_data(client, listener, context)
 
                 # Ensure upload_cert is called three times
                 calls_cert_mngr = [
-                    mock.call.get_cert('12345', 'cont_id_1', check_only=True),
-                    mock.call.get_cert('12345', 'cont_id_2', check_only=True),
-                    mock.call.get_cert('12345', 'cont_id_3', check_only=True)
+                    mock.call.get_cert(context, 'cont_id_1', check_only=True),
+                    mock.call.get_cert(context, 'cont_id_2', check_only=True),
+                    mock.call.get_cert(context, 'cont_id_3', check_only=True)
                 ]
                 client.assert_has_calls(calls_cert_mngr)
 
+        # Test asking for nothing
+        listener = sample_configs_combined.sample_listener_tuple(
+            tls=False, sni=False, client_ca_cert=False)
+        client = mock.MagicMock()
+        with mock.patch.object(cert_parser,
+                               '_map_cert_tls_container') as mock_map:
+            result = cert_parser.load_certificates_data(client, listener)
+
+            mock_map.assert_not_called()
+            ref_empty_dict = {'tls_cert': None, 'sni_certs': []}
+            self.assertEqual(ref_empty_dict, result)
+            mock_oslo.assert_called()
+
     @mock.patch('octavia.certificates.common.cert.Cert')
     def test_map_cert_tls_container(self, cert_mock):
         tls = data_models.TLSContainer(
+            id=sample_certs.X509_CERT_SHA1,
             primary_cn=sample_certs.X509_CERT_CN,
             certificate=sample_certs.X509_CERT,
             private_key=sample_certs.X509_CERT_KEY_ENCRYPTED,
@@ -163,6 +192,9 @@ class TestTLSParseUtils(base.TestCase):
         cert_mock.get_private_key_passphrase.return_value = tls.passphrase
         with mock.patch.object(cert_parser, 'get_host_names') as cp:
             cp.return_value = {'cn': sample_certs.X509_CERT_CN}
+            self.assertEqual(
+                tls.id, cert_parser._map_cert_tls_container(
+                    cert_mock).id)
             self.assertEqual(
                 tls.primary_cn, cert_parser._map_cert_tls_container(
                     cert_mock).primary_cn)
@@ -179,7 +211,7 @@ class TestTLSParseUtils(base.TestCase):
 
     def test_build_pem(self):
         expected = b'imacert\nimakey\nimainter\nimainter2\n'
-        tls_tuple = sample_configs.sample_tls_container_tuple(
+        tls_tuple = sample_configs_combined.sample_tls_container_tuple(
             certificate=b'imacert', private_key=b'imakey',
             intermediates=[b'imainter', b'imainter2'])
         self.assertEqual(expected, cert_parser.build_pem(tls_tuple))
